unit Unit1;

interface

uses
  System.SysUtils, System.Types, System.UITypes, System.Classes, System.Variants,
  FMX.Types, FMX.Controls, FMX.Forms, FMX.Dialogs, FMX.Filter.Effects,
  FMX.Objects, FMX.Ani, FMX.Effects;

type
  TFormMain = class(TForm)
    btnIntro: TButton;
    btnNext: TButton;
    CalloutPanel1: TCalloutPanel;
    CalloutPanel2: TCalloutPanel;
    Panel1: TPanel;
    ArcDial1: TArcDial;
    ArcDial2: TArcDial;
    Track1: TTrack;
    btnClose: TButton;
    ImageControl1: TImageControl;
    tmrStart: TTimer;
    PanelGraph: TPanel;
    PixelateEffect1: TPixelateEffect;
    TrackBar1: TTrackBar;
    ArcDial3: TArcDial;
    procedure FormCreate(Sender: TObject);
    procedure tmrStartTimer(Sender: TObject);
    procedure btnIntroClick(Sender: TObject);
    procedure btnNextClick(Sender: TObject);
    procedure btnCloseClick(Sender: TObject);
  private
    Bitmap1 :TBitmap;
    Bitmap2 :TBitmap;
    procedure cmdResetControls;
    procedure cmdRandomizeGraph;
    procedure cmdAnimateGraph(interval: Extended);
    { Private declarations }
  public
    { Public declarations }
  end;

var
  FormMain: TFormMain;

implementation

{$R *.fmx}

procedure TFormMain.btnCloseClick(Sender: TObject);
begin
  Close;
end;

procedure TFormMain.btnIntroClick(Sender: TObject);
begin
  cmdResetControls;
  tmrStart.Interval := 500;
  tmrStart.Enabled := True;
end;

procedure TFormMain.cmdResetControls;
var
  i: Integer;
begin
  CalloutPanel1.Visible := False;
  CalloutPanel2.Visible := False;
  Panel1.Visible := False;
  PanelGraph.Visible := False;

  CalloutPanel1.Opacity := 0;
  CalloutPanel2.Opacity := 0;
  Panel1.Opacity := 0;
  PanelGraph.Opacity := 0;

  for i := 0 to PanelGraph.ChildrenCount-1 do
    if PanelGraph.Children[i] is TControl then
      (PanelGraph.Children[i] as TControl).Width := 0;
end;

procedure TFormMain.btnNextClick(Sender: TObject);
var
  interval: Extended;
  id: Integer;
begin
  interval := 1.5;
  cmdRandomizeGraph;
  cmdAnimateGraph( interval );
  // 50 - 16
  PixelateEffect1.BlockCount := 50;
  PixelateEffect1.Enabled := True;
  PixelateEffect1.AnimateFloatWait('BlockCount',10,interval / 2);
  id := 1-ImageControl1.Tag;
  case id of
    0: ImageControl1.Bitmap.Assign(Bitmap1);
    1: ImageControl1.Bitmap.Assign(Bitmap2);
  end;
  PixelateEffect1.AnimateFloatWait('BlockCount',50,interval / 2);
  PixelateEffect1.Enabled := False;
  ImageControl1.Tag := id;
end;

// ------------------------------- ------------------------------- ------
// ------------------------------- ------------------------------- ------

const
  MARGIN = 5;

procedure TFormMain.cmdRandomizeGraph;
var
  i: Integer;
  max_elem_wd: Int64;
  r: TRectangle;
begin
  max_elem_wd := round(PanelGraph.Width-2*MARGIN);
  for i := 0 to PanelGraph.ChildrenCount-1 do
    if PanelGraph.Children[i] is TControl then begin
      r := PanelGraph.Children[i] as TRectangle;
      r.Tag := 50 + Random(max_elem_wd-50);
    end;
end;

procedure TFormMain.cmdAnimateGraph(interval: Extended);
var
  i: Integer;
  c: TControl;
begin
  for i := 0 to PanelGraph.ChildrenCount - 1 do
    if PanelGraph.Children[i] is TControl then begin
      c := PanelGraph.Children[i] as TControl;
      c.AnimateFloat('Width', c.Tag, interval);
    end;
end;

procedure TFormMain.FormCreate(Sender: TObject);
var
  elems: Int64;
  i: Integer;
  r: TRectangle;
  elem_hg: Integer;
  stm: TResourceStream;
begin
  Randomize;
  // ----------------------------
  // Wypenienie Bitmap1 i Bitmap2 (z ImageControl i z zasobw)
  Bitmap1 := TBitmap.Create(0,0);
  Bitmap1.Assign( ImageControl1.Bitmap );
  Bitmap2 := TBitmap.Create(0,0);
  stm := TResourceStream.Create(hInstance, 'JpgImage_2', RT_RCDATA );
  Bitmap2.LoadFromStream(stm);
  stm.Free;
  // -----------------------------
  // "Wyzerowanie" i ukrycie kontrolek przed animacj
  cmdResetControls;
  // -----------------------------
  // stworzenie wykresu z komponentw TRectangle
  elem_hg := 13;
  elems := round(PanelGraph.Height-2*MARGIN) div elem_hg;
  for i := 0 to elems-1 do begin
    r := TRectangle.Create(self);
    r.Position.X := MARGIN;
    r.Position.Y := MARGIN + elem_hg*i;
    r.Height := elem_hg+1;
    r.Parent := PanelGraph;
    r.Fill.Color := $ffffff00;
  end;
  // -----------------------------
  // losowy ukad wykresu
  cmdRandomizeGraph;
end;

// ------------------------------- ------------------------------- ------
// ------------------------------- ------------------------------- ------

procedure TFormMain.tmrStartTimer(Sender: TObject);
var
  interval: Extended;
  y1: Extended;
  y2: Extended;
  x1: Extended;
  y3: Extended;
begin
  tmrStart.Enabled := False;
  // -------------------------------
  // czas trwania animacji INTRO
  interval := 1.5;
  // -------------------------------
  // "Pokazanie" kontrolek - nie wida ich bo maj Opacity=0
  CalloutPanel1.Visible := True;
  CalloutPanel2.Visible := True;
  Panel1.Visible := True;
  PanelGraph.Visible := True;
  // -------------------------------
  // sprawdzenie pozycji biecych kontrolek
  y1 := CalloutPanel1.Position.Y;
  y2 := CalloutPanel2.Position.Y;
  x1 := Panel1.Position.X;
  y3 := PanelGraph.Position.Y;
  // -------------------------------
  // przesunicie kontrolek prze animacj
  CalloutPanel1.Position.Y := CalloutPanel1.Position.Y - 100;
  CalloutPanel2.Position.Y := CalloutPanel2.Position.Y + 100;
  PanelGraph.Position.Y := PanelGraph.Position.Y + 100;
  Panel1.Position.X := Panel1.Position.X + Panel1.Width/2;
  // -------------------------------
  // zdefiniowanie animacji przezroczystoci: 0 .. 1
  CalloutPanel1.AnimateFloat('Opacity', 1, interval);
  CalloutPanel2.AnimateFloat('Opacity', 1, interval);
  Panel1.AnimateFloat('Opacity', 1, interval);
  PanelGraph.AnimateFloat('Opacity', 1, interval);
  // -------------------------------
  // zdefiniowanie animacji ruchu (pozycja x lub pozycja y)
  CalloutPanel1.AnimateFloat('Position.Y', y1, interval);
  CalloutPanel2.AnimateFloat('Position.Y', y2, interval);
  Panel1.AnimateFloat('Position.X', x1, interval);
  PanelGraph.AnimateFloat('Position.Y', y3, interval);
  // -------------------------------
  // dodaenie animacji do wykresu
  cmdAnimateGraph(interval);
end;

end.
